/***************************************************************************
                          knetworkconf.cpp  -  description
                             -------------------
    begin                : Sun Jan 12 8:54:19 UTC 2003
    copyright            : (C) 2003 by Juan Luis Baptiste
    email                : jbaptiste@merlinux.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "knetworkconf.h"

KNetworkConf::KNetworkConf(QWidget *parent, const char *name) : KNetworkConfDlg(parent, name)
{
  makeButtonsResizeable();
  config = new KNetworkConfigParser();
  connect(config,SIGNAL(readyLoadingNetworkInfo()),this,SLOT(loadNetworkInfo()));
  connect(config,SIGNAL(readyLoadingNetworkInfo()),this,SLOT(showMainWindow()));  
  connect(config,SIGNAL(network_reloaded()),this,SLOT(updateWidgets()));
  connect(config, SIGNAL(readyLoadingNetworkInfo()),this,SLOT(updateNetworkDevicesInfo()));
}

KNetworkConf::~KNetworkConf()
{
}

void KNetworkConf::loadNetworkInfo()
{
  netInfo = config->loadNetworkInfo3();
  if (netInfo == NULL)
  {
    KMessageBox::error(this,
                        i18n("Couldn't load Network Info."),
                        i18n("Error reading Configuration File"));
    //kapp->quit();
    exit(1);
  }
  else
  {
    deviceList.clear();
    deviceList = netInfo->getDeviceList();
    routingInfo = netInfo->getRoutingInfo();
    dnsInfo = netInfo->getDNSInfo();

    loadNetworkDevicesInfo();
    loadRoutingAndDNSInfo();
    loadDNSInfoTab();
    nameServersModified = false;
    devicesModified = false;
    modified = false;
    kpbApply->setEnabled(false);
  }

}  
void KNetworkConf::loadNetworkDevicesInfo()
{
//  deviceList = config->loadNetworkDevices();
  //Checks the state of the configured devices
  paintListView();
}
void KNetworkConf::paintListView()
{
  KNetworkInterface *device;
  QPixmap deviceImg(locate("icon","network_card.png"));
  klvCardList->clear();
  for (device = deviceList.first(); device; device = deviceList.next())
  {
    if (klvCardList->findItem(device->getDeviceName(),CaseSensitive|ExactMatch) == 0)
    {
      QListViewItem * item = new QListViewItem( klvCardList, 0 );

      item->setPixmap( 0, deviceImg);
      item->setText(0,device->getDeviceName());
      item->setText(1,device->getIpAddress());
      item->setText(2,device->getBootProto());
      item->setText(4,device->getDescription());
      deviceNamesList.append(device->getDeviceName());
    }  
  }

}  
void KNetworkConf::updateNetworkDevicesInfo()
{
  netInfo = config->loadNetworkInfo3();
  KNetworkInterface *device;
  for (device = deviceList.first(); device; device = deviceList.next())
  {
//    if (klvCardList->findItem(device->getDeviceName(),0,ExactMatch) == 0)
    {
      QListViewItem *item = klvCardList->findItem(device->getDeviceName(),0,ExactMatch);
      if (item != NULL)
      {
        item->setText(1,device->getIpAddress());
        item->setText(2,device->getBootProto());
      }
    }  
  }
  //Checks the state of the configured devices
  updateDevicesState();
}
void KNetworkConf::updateDhcpDevicesInfo()
{
  KNetworkInterface *device;
  klvCardList->clear();
  for (device = deviceList.first(); device; device = deviceList.next())
  {
//    if (klvCardList->findItem(device->getDeviceName(),0,ExactMatch) == 0)
    {
      QListViewItem *item = klvCardList->findItem(device->getDeviceName(),0,ExactMatch);
      item->setText(1,device->getIpAddress());
      item->setText(2,device->getBootProto());
    }  
  }
  //Checks the state of the configured devices
    updateDevicesState();
  
}

/** Terminates the application
 */
void KNetworkConf::quitSlot(){
  int code = 0;
  if (modified)
  {
    code = KMessageBox::questionYesNo(this,
                          i18n("The new configuration hasn't been saved.\n Do you want to apply changes before quitting?"),
                          i18n("New configuration not saved."));
    if (code == KMessageBox::Yes)
      saveInfoSlot();
    else
      kapp->quit();
  }
  else                        
    kapp->quit();
}
/** Enables the configure and remove buttons. */
void KNetworkConf::enableButtonsSlot(){
  if (!readOnly)
  {
     kpbConfigureNetworkInterface->setEnabled(true);
     kpbUpDownButton->setEnabled(true);
  }   
}
/** opens the add server dialog. */
void KNetworkConf::addServerSlot(){
  KAddDNSServerDlg addDlg(this, 0);
  addDlg.exec();
  if(!(addDlg.kleNewServer->text().isEmpty()))
  {
    klbDomainServerList->insertItem(addDlg.kleNewServer->text());
    nameServersModified = true;
    enableApplyButtonSlot();
    modified = true;
  }
}

/** Pops up the window for adding a new interface. */
void KNetworkConf::configureDeviceSlot(){
  KAddDeviceDlg addDlg(this, 0);
  QListViewItem *item = klvCardList->currentItem();
  QString currentDevice = item->text(0);
  addDlg.setCaption(i18n("Configure device "+currentDevice));
  KNetworkInterface *dev = getDeviceInfo(currentDevice);
  if (dev->getBootProto() == "none")
    addDlg.kcbBootProto->setCurrentItem(0);
  else if (dev->getBootProto().lower() == "dhcp")
  {
    addDlg.kcbBootProto->setCurrentItem(1);
    addDlg.gbBasicDeviceInfo->setEnabled(false);
    addDlg.kpbAdvanced->setEnabled(false);
  }
  else if (dev->getBootProto().lower() == "bootp")
  {
    addDlg.kcbBootProto->setCurrentItem(2);
    addDlg.gbBasicDeviceInfo->setEnabled(false);
    addDlg.kpbAdvanced->setEnabled(false);
  }
  if (dev->getOnBoot() == "yes")
    addDlg.kcbstartAtBoot->setChecked(true);
  else
  {
    addDlg.kcbstartAtBoot->setChecked(false);
  }
  addDlg.kleIPAddress->setText(dev->getIpAddress());
  addDlg.kleDescription->setText(dev->getDescription());
  addDlg.kleNetwork->setText(dev->getNetwork());
  addDlg.kleBroadcast->setText(dev->getBroadcast());

  if (!dev->getNetmask().isEmpty())
    addDlg.kcbNetmask->setCurrentText(dev->getNetmask());

  if (readOnly)
  {
    addDlg.gbOptions->setEnabled(false);
    addDlg.gbBasicDeviceInfo->setEnabled(false);    
  }
  addDlg.kpbApply->setEnabled(false);

  addDlg.exec(); //show window

  if (addDlg.modified())
  {
    item->setText(1,addDlg.kleIPAddress->text());
    item->setText(2,addDlg.kcbBootProto->currentText());
    item->setText(4,addDlg.kleDescription->text());

    dev->setBootProto(addDlg.kcbBootProto->currentText());
    dev->setDescription(addDlg.kleDescription->text());
    //If the selected boot protocol is dhcp, then we don't need the
    //past IP address and netmask.
    if (addDlg.kcbBootProto->currentText().lower().compare("dhcp") == 0)
    {
      dev->setIpAddress("");
      dev->setNetmask("");
      dev->setNetwork("");
      dev->setBroadcast("");
      item->setText(2,"");
      item->setText(3,"");      
      enableApplyButtonSlot();
      modified = true;      
    }
    else
    { //Only update the device object and set modified=true if the values changed
      if (valuesChanged(dev,
                        addDlg.kcbBootProto->currentText(),
                        addDlg.kcbNetmask->currentText(),
                        addDlg.kleIPAddress->text(),
                        addDlg.kcbstartAtBoot->isChecked(),
                        addDlg.kleDescription->text()))
      {
        dev->setIpAddress(addDlg.kleIPAddress->text().stripWhiteSpace());
        dev->setNetmask(addDlg.kcbNetmask->currentText().stripWhiteSpace());
        QString network = KAddressValidator::calculateNetwork(dev->getIpAddress().stripWhiteSpace(),dev->getNetmask().stripWhiteSpace());
        dev->setNetwork(network);        
        QString broadcast = KAddressValidator::calculateBroadcast(dev->getIpAddress().stripWhiteSpace(),dev->getNetmask().stripWhiteSpace());
        dev->setBroadcast(broadcast);
        dev->setDescription(addDlg.kleDescription->text());

        if (addDlg.kcbstartAtBoot->isChecked())
          dev->setOnBoot("yes");
        else
          dev->setOnBoot("no");
      }
      devicesModified = true;
      modified = true;
      enableApplyButtonSlot();
    }
  }
}
/**Returns the info of the network device 'device or NULL if not found.'*/
KNetworkInterface * KNetworkConf::getDeviceInfo(QString device){
  QPtrListIterator<KNetworkInterface>  i(deviceList);
  KNetworkInterface *temp;
  while ((temp = i.current()) != 0)
  {
    if (temp->getDeviceName() == device)
    {
      return temp;
    }
    ++i;
  }
  return NULL;
}
/**Returns the name of the network device that corresponds to the IP address 'ipAddr' or NULL if not found.'*/
QString KNetworkConf::getDeviceName(QString ipAddr){
  QPtrListIterator<KNetworkInterface>  i(deviceList);
  KNetworkInterface *temp;
  while ((temp = i.current()) != 0)
  {
    if (temp->getIpAddress().compare(ipAddr) == 0)
    {
      return temp->getDeviceName();
    }
    ++i;
  }
  return NULL;
}


void KNetworkConf::updateDevicesState(){
  procUpdateDevice = new QProcess(this);
//  isUp = false;
  procUpdateDevice->addArgument( "/sbin/ifconfig" );

  connect( procUpdateDevice, SIGNAL(readyReadStdout()),
                this, SLOT(readFromStdout()) );
  connect( procUpdateDevice, SIGNAL(processExited()),
                this, SLOT(changeState()) );

  if ( !procUpdateDevice->start() )
  {
    // error handling
    KMessageBox::error(this,
                        i18n("Could not launch command ifconfig"),
                        i18n("There was an error executing external program"));

  }
}
/** Looks in the output returned by ifconfig to see if there are the devices up or down.*/
void KNetworkConf::readFromStdout(){
  commandOutput = "";
  commandOutput += procUpdateDevice->readStdout();
}
void KNetworkConf::changeState(){
  KNetworkInterface *dev;
  for ( QStringList::Iterator it = deviceNamesList.begin(); it != deviceNamesList.end(); ++it)
  {
    dev = getDeviceInfo(*it);
    if (isDeviceActive(dev->getDeviceName(),commandOutput))
    {
      QListViewItem *item = klvCardList->findItem(*it,0,ExactMatch);
      dev = getDeviceInfo(*it);
      dev->setActive(true);
      item->setText(3,"Up");
    }
    else
    {
      QListViewItem *item = klvCardList->findItem(*it,0,ExactMatch);
      dev = getDeviceInfo(*it);
      dev->setActive(false);
      item->setText(3,"Down");
    }
  }
}
/** Loads the info about the default gateway and host and domain names. */
void KNetworkConf::loadRoutingAndDNSInfo(){
  //routingInfo = config->loadNetworkInfo();
  if (routingInfo == NULL)
    KMessageBox::error(this,i18n("Couldn't open file '/etc/sysconfig/entwork' for reading!"),
                        i18n("There was an error loading the config files"));
  else
  {
    kleDefaultRoute->setText(routingInfo->getGateway());
    kcbGwDevice->clear();    
    kcbGwDevice->insertStringList(deviceNamesList);
    if (!routingInfo->getGatewayDevice().isEmpty())    
      kcbGwDevice->setCurrentText(routingInfo->getGatewayDevice());
    if (routingInfo->isForwardIPv4Enabled().compare("yes") == 0)
      cbEnableRouting->setChecked(true);
    else
      cbEnableRouting->setChecked(false);
  }
}
void KNetworkConf::loadDNSInfoTab(){
  QStringList nameServers;
  if (dnsInfo == NULL)
    KMessageBox::error(this,i18n("Couldn't open file '/etc/resolv.conf' for reading!"),
                        i18n("There was an error loading the config files"));
  else
  {
    kleHostName->setText(dnsInfo->getMachineName());
    kleDomainName->setText(dnsInfo->getDomainName());    
    klbDomainServerList->clear();
    nameServers = dnsInfo->getNameServers();
    for ( QStringList::Iterator it = nameServers.begin(); it != nameServers.end(); ++it)
    {
      klbDomainServerList->insertItem(*it);
    }
    klvKnownHosts->clear();
    knownHostsList = dnsInfo->getKnownHostsList();
    QPtrListIterator<KKnownHostInfo> it(knownHostsList);    
    KKnownHostInfo *host;
    while ((host = it.current()) != 0)
    {
      ++it;
      QListViewItem * item = new QListViewItem( klvKnownHosts, 0 );

      item->setText(0,host->getIpAddress());
      item->setText(1,host->getHostName());
      item->setText(2,host->getAliases());
    }  
  }  
}
/** Shows the help browser. Hopefully some day it will be one :-). */
void KNetworkConf::helpSlot(){
  kapp->invokeHelp();
}
/** No descriptions */
void KNetworkConf::aboutSlot(){
  KAboutApplication *about = new KAboutApplication(kapp->aboutData(),0);
//  about->setLogo(QPixmap(QDir::homeDirPath()+"/Proyectos_KDevelop/knetworkconf/knetworkconf/lo32-app-knetworkconf.png"));
//  about->setLogo(QDir::homeDirPath()+"/Proyectos_KDevelop/knetworkconf/knetworkconf/lo32-app-knetworkconf.png");

  about->show();
}
/** No descriptions */
void KNetworkConf::enableApplyButtonSlot(){
 if (!readOnly)
    kpbApply->setEnabled(true);

}
/** Puts the application in read-only mode. This happens when the user runing
the application isn't root. */
void KNetworkConf::setReadOnly(bool state){
  KNetworkConf::readOnly = state;
}
/** No descriptions */
void KNetworkConf::enableApplyButtonSlot(const QString &text){
  enableApplyButtonSlot();
}
/** No descriptions */
void KNetworkConf::enableApplyButtonSlot(bool){
  enableApplyButtonSlot();
}
/** No descriptions */
void KNetworkConf::removeServerSlot(){
  klbDomainServerList->removeItem(klbDomainServerList->currentItem());
  modified = true;
  enableApplyButtonSlot();
}
void KNetworkConf::moveUpServerSlot(){
  int curPos = klbDomainServerList->currentItem();
  int antPos = klbDomainServerList->currentItem() - 1;

  if (antPos >= 0)
  {
    QListBoxItem *current = klbDomainServerList->item(curPos);
    QListBoxItem *ant = current->prev();;
    QString antText = ant->text();
    klbDomainServerList->removeItem(antPos);
    klbDomainServerList->insertItem(antText,curPos);
    modified = true;
  }
}
void KNetworkConf::moveDownServerSlot(){
  unsigned curPos = klbDomainServerList->currentItem();
  unsigned nextPos = klbDomainServerList->currentItem() + 1;
  
  if (klbDomainServerList->count() >= nextPos)
  {
    QListBoxItem *current = klbDomainServerList->item(curPos);
    QString curText = current->text();
    klbDomainServerList->removeItem(curPos);
    klbDomainServerList->insertItem(curText,nextPos);
    klbDomainServerList->setSelected(nextPos,true);
    modified = true;    
  }  
}
/** Disables all buttons a line edit widgets when the user has read only access. */
void KNetworkConf::disableAll(){
  kleHostName->setReadOnly(true);
  kleDomainName->setReadOnly(true);
  tlDomainName->setEnabled(false);
  tlHostName->setEnabled(false);  
  disconnect(klvCardList,SIGNAL(doubleClicked(QListViewItem *)),this,SLOT(configureDeviceSlot()));
  gbNetworkDevices->setEnabled(false);
  gbDefaultGateway->setEnabled(false);
  gbDNSServersList->setEnabled(false);
  gbKnownHostsList->setEnabled(false);
//  gbNetworkOptions->setEnabled(false);
}
/** Saves all the modified info of devices, routes,etc. */
void KNetworkConf::saveInfoSlot(){
  config->setProgramVersion(getVersion());

  if (!KAddressValidator::isValidIPAddress(kleDefaultRoute->text()) && (kleDefaultRoute->text() != ""))
  {
    KMessageBox::error(this,i18n("The default Gateway IP address is invalid."),i18n(" Invalid IP address"));
  }
  else
  {
    //Update DNS info
    routingInfo->setDomainName(kleDomainName->text());
    routingInfo->setHostName(kleHostName->text());
    dnsInfo->setDomainName(kleDomainName->text());
    dnsInfo->setMachineName(kleHostName->text());
    
    dnsInfo->setNameServers(getNamserversList(klbDomainServerList));
    dnsInfo->setKnownHostsList(getKnownHostsList(klvKnownHosts));
    //Update routing info
    routingInfo->setGateway(kleDefaultRoute->text());
    if (routingInfo->getGateway().isEmpty())
      routingInfo->setGatewayDevice("");
    
    if (cbEnableRouting->isChecked())
      routingInfo->setForwardIPv4Enabled("yes");
    else
      routingInfo->setForwardIPv4Enabled("no");

    if (kleDefaultRoute->text() != "")
      routingInfo->setGatewayDevice(kcbGwDevice->currentText());


    //Save all info
    netInfo->setDeviceList(deviceList);
    netInfo->setRoutingInfo(routingInfo);
    netInfo->setDNSInfo(dnsInfo);
    if (!config->saveNetworkInfo3(netInfo))
      KMessageBox::error(this,
                         i18n("The application was unable to save one of the configuration files.\nPlease review file permissions."),
                         i18n("Unable to save configuration"));
    else
    {
       kpbApply->setEnabled(false);
       modified = false;
    }
  }      
}
/** Creates a QStringList with the IP addresses contained in the QListBox of name servers. */
QStringList KNetworkConf::getNamserversList(KListBox * serverList){
  QStringList list;
  for (unsigned i = 0; i < serverList->count(); i++)
  {
    list.append(serverList->text(i));
  }
  return list;  
}
/** Creates a QPtrList<KKownHostInfo> with the info contained in the KListView of name servers. */
QPtrList<KKnownHostInfo> KNetworkConf::getKnownHostsList(KListView * hostsList){
  QPtrList<KKnownHostInfo> list;
  QListViewItem *it = hostsList->firstChild();         
  for (int i = 0; i < hostsList->childCount(); i++)
  {
    KKnownHostInfo *host = new KKnownHostInfo();
  //  it = hostsList->currentItem();
    host->setIpAddress(it->text(0));
    host->setHostName(it->text(1));
    host->setAliases(it->text(2));
    it = it->nextSibling();
    list.append(host);
  }
  return list;
}

QString KNetworkConf::getVersion(){
  return version;
}
void KNetworkConf::setVersion(QString ver){
  KNetworkConf::version = ver;
}

/** Changes the state of device 'dev' to DEVICE_UP or DEVICE_DOWN.
Return true on success, false on failure.  */
void KNetworkConf::changeDeviceState(QString dev, int state){
  KInterfaceUpDownDlg* dialog = new KInterfaceUpDownDlg(0,"Changing device state");
  if (state == DEVICE_UP)
    dialog->label->setText(i18n("Bringing up interface <b>" + dev + "</b>..."));
  else
    dialog->label->setText(i18n("Bringing down interface <b>" + dev + "</b>..."));
  dialog->show();

  procDeviceState = new QProcess(this);
  QString cmd;
  procDeviceState->addArgument( locate("data",BACKEND_PATH) );
  
  //If the platform couldn't be autodetected specify it manually
  if (netInfo->getPlatformName() != QString::null)
  {
    procDeviceState->addArgument( "--platform" );    
    procDeviceState->addArgument( netInfo->getPlatformName() );
  }  
  procDeviceState->addArgument( "-d" );

  if (state == DEVICE_UP)
    procDeviceState->addArgument("enable_iface::"+dev+"::1" );
  else if (state == DEVICE_DOWN)
    procDeviceState->addArgument("enable_iface::"+dev+"::0" );

  connect(dialog->cancelButton, SIGNAL(clicked()), procDeviceState, SLOT(tryTerminate()));
  connect( dialog->cancelButton, SIGNAL(clicked()), dialog, SLOT(close()) );

  connect( procDeviceState, SIGNAL(processExited()), dialog, SLOT(close()) );
  connect( procDeviceState, SIGNAL(readyReadStdout()),this, SLOT(readFromStdoutUpDown()) );
  connect( procDeviceState, SIGNAL(readyReadStderr()),this, SLOT(readFromStdErrUpDown()) );
//  connect( procDeviceState, SIGNAL(processExited()),this, SLOT(updateDevicesState()) );
  connect( procDeviceState, SIGNAL(processExited()),this, SLOT(verifyDeviceStateChanged()) );
  currentDevice = dev;
  commandOutput = "";

  if ( !procDeviceState->start() )
  {
    // error handling
    KMessageBox::error(this,
                        i18n("couldn't launch backend to change network device state. You will have to do it manually. "),
                        i18n("Error "));
    dialog->close();
//    return false;
  }

}
void KNetworkConf::readFromStdoutUpDown(){
  commandOutput.append(procDeviceState->readStdout());
}

void KNetworkConf::verifyDeviceStateChanged(){
  KNetworkInterface *dev;

  commandOutput = commandOutput.section('\n',1);
  if (commandErrOutput.length() > 0)
  {
        KMessageBox::error(this,
                            i18n("There was an error changing the device's state. You will have to do it manually."),
                            i18n("Couldn't change device state"));

  }
  else if (commandOutput == "\n<!-- GST: end of request -->")
  {
    QListViewItem *item = klvCardList->findItem(currentDevice,0,ExactMatch);
    if (item != NULL)
    {
      dev = getDeviceInfo(currentDevice);
      if (!dev->isActive())
      {
        dev->setActive(true);
        item->setText(3,"Up");
//        config->runDetectionScript(netInfo->getPlatformName());
        config->listIfaces(netInfo->getPlatformName());
//        item->setText(1,dev->getIpAddress()); 
      }
      else
      {
        dev->setActive(false);
        item->setText(3,"Down");
        if (dev->getBootProto().lower() == "dhcp")
          item->setText(1,"");        
      }
    }
  }
}
/** Returns a list of strings of all the configured devices. */
QStringList KNetworkConf::getDeviceList(){
  QStringList list;
  KNetworkInterface * device;
  for (device = deviceList.first(); device; device = deviceList.next())
  {
    list.append(device->getDeviceName());
  }
  return list;
}
/** No descriptions */
bool KNetworkConf::valuesChanged(KNetworkInterface *dev,
                                  QString bootProto,
                                  QString netmask,
                                  QString ipAddr,
                                  bool onBoot,
                                  QString desc){
  if ((dev->getBootProto() != bootProto) ||
      (dev->getNetmask() != netmask) ||
      (dev->getIpAddress() != ipAddr) ||
      ((dev->getOnBoot() == "yes") && !(onBoot)) ||
      ((dev->getOnBoot() == "no") && (onBoot)) ||
      (dev->getDescription() != desc))
    return true;
  else
    return false;          
}
/** No descriptions */
void KNetworkConf::updateWidgets(){
  //get the network info again to see the changes of dhcp devices.
  netInfo = config->loadNetworkInfo3();
  updateNetworkDevicesInfo();
  loadRoutingAndDNSInfo();
  loadDNSInfoTab();  
}
/** Sets the QPushButton::autoResize() in true for all buttons. */
void KNetworkConf::makeButtonsResizeable(){
  kpbHelp->setAutoResize(true);
  kpbAbout->setAutoResize(true);
  kpbApply->setAutoResize(true);
  kpbQuit->setAutoResize(true);
  kpbConfigureNetworkInterface->setAutoResize(true);
  kcbGwDevice->setAutoResize(true);
  kpbAddDomainServer->setAutoResize(true);
  kpbRemoveDomainServer->setAutoResize(true);
  kpbUpDownButton->setAutoResize(true);
  kpbAddKnownHost->setAutoResize(true);
  kpbRemoveKnownHost->setAutoResize(true);
}
void KNetworkConf::upDownInterfaceSlot()
{
  KNetworkInterface *dev = getDeviceInfo(klvCardList->currentItem()->text(0));
  if (dev->isActive())
    changeDeviceState(dev->getDeviceName(),DEVICE_DOWN);
  else
    changeDeviceState(dev->getDeviceName(),DEVICE_UP);
}
/** Adds a new host to the KListView that has the known hosts. */
void KNetworkConf::addKnownHostSlot(){
  KAddKnownHostDlg dlg(this,0);
  dlg.setCaption("Add a new known Host");
  dlg.exec();
  if (!(dlg.kleIpAddress->text().isEmpty() && dlg.kleName->text().isEmpty()))
  {
    QListViewItem * item = new QListViewItem( klvKnownHosts, 0 );

    item->setText(0,dlg.kleIpAddress->text());
    item->setText(1,dlg.kleName->text());
    item->setText(2,dlg.kleAliases->text());
    enableApplyButtonSlot();
    modified = true;
  }
}
/** Removes a known host from the list view */
void KNetworkConf::removeKnownHostSlot(){
  klvKnownHosts->removeItem(klvKnownHosts->currentItem());
}
/** Edits the info about a known host. */
void KNetworkConf::editKnownHostSlot(){
  KAddKnownHostDlg dlg(this, 0);
  dlg.setCaption("Edit Known Host Info");
  dlg.kpbAdd->setText("Edit");
  QListViewItem *item = klvKnownHosts->currentItem();
  dlg.kleIpAddress->setText(item->text(0));
  dlg.kleName->setText(item->text(1));
  dlg.kleAliases->setText(item->text(2));
  dlg.exec();
  if (dlg.modified())
  {
    item->setText(0,dlg.kleIpAddress->text());
    item->setText(1,dlg.kleName->text());
    item->setText(2,dlg.kleAliases->text());
    
  }
}
/** Shows the main window after the network info has been loaded. */
void KNetworkConf::showMainWindow(){
  this->show();
}
/** No descriptions */
void KNetworkConf::readFromStdErrUpDown(){
  commandErrOutput.append(procDeviceState->readStderr());
}
/** Tries to stop the network. */
void KNetworkConf::stopNetworkSlot(){
  for ( QStringList::Iterator it = deviceNamesList.begin(); it != deviceNamesList.end(); ++it)
  {
    changeDeviceState(*it,DEVICE_DOWN);
  }
}
/** Tries to start the network. */
void KNetworkConf::startNetworkSlot(){
  for ( QStringList::Iterator it = deviceNamesList.begin(); it != deviceNamesList.end(); ++it)
  {
//    changeDeviceState("eth0",DEVICE_UP);
    changeDeviceState(*it,DEVICE_UP);
//    sleep(2);
  }
}
/** No descriptions */
void KNetworkConf::loadAgainNetworkInfoSlot(){
  config->loadNetworkInfo3();
}
/** No descriptions */
void KNetworkConf::getNewNetworkInfo(){
  klvCardList->clear();
  netInfo = config->getNewNetworkInfo();
  deviceList = netInfo->getDeviceList();
  routingInfo = netInfo->getRoutingInfo();
  dnsInfo = netInfo->getDNSInfo();
  loadNetworkDevicesInfo();
  loadRoutingAndDNSInfo();
  loadDNSInfoTab();
  updateDhcpDevicesInfo();
}
/** Sees if a device is active or not in the ifconfig output. Not very nice, but it works. Inthe future, this has to be managed by gst. */
bool KNetworkConf::isDeviceActive(QString device, QString ifconfigOutput){
    QString temp = ifconfigOutput.section(device,1,1);
    if (temp == "")
      return false;
    else
    {  
      QString temp2 = temp.section("UP",0,0); //two firts lines of the device info.
      QString temp3 = temp2.section("\n",0,0); //Link encap:Ethernet  HWaddr 00:00:21:C5:99:A0
      QString temp4 = temp2.section("\n",1,1); //inet addr:192.168.1.1  Bcast:192.255.255.255  Mask:255.0.0.0
      temp3 = temp3.stripWhiteSpace();
      temp4 = temp4.stripWhiteSpace();
      QString temp5 = temp3.section(" ",4,4); //00:00:21:C5:99:A0
      QString temp6 = temp4.section(" ",1,1); // addr:192.168.1.1
      temp6 = temp6.section(":",1,1); //192.168.1.1
      QString temp7 = temp4.section(" ",3,3); //Bcast:192.255.255.255
      temp7 = temp7.section(":",1,1); //192.255.255.255
      QString temp8 = temp4.section(" ",5,5); // Mask:255.0.0.0
      temp8 = temp8.section(":",1,1); //255.0.0.0

      //If the ip address is empty it must be a dhcp interface, so fill these fields:
      if (temp6.isEmpty())
        return false;
    }
    return true;
}
