/***************************************************************************
                          knetworkconfigparser.h  -  description
                             -------------------
    begin                : Mon Jan 13 2003
    copyright            : (C) 2003 by Juan Luis Baptiste
    email                : jbaptiste@merlinux.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KNETWORKCONFIGPARSER_H
#define KNETWORKCONFIGPARSER_H
#define PATH_TO_RH_IFCFG_ETHx "/etc/sysconfig/network-scripts/"
#define PATH_TO_RH_NETWORK_INFO "/etc/sysconfig/network"
#define PATH_TO_RESOLV_CONF "/etc/resolv.conf"
#define PATH_TO_CONF_FILE "/etc/knetwork.conf"
#define PATH_TO_HOSTS "/etc/hosts"
#define PATH_TO_RH_RELOAD_SCRIPT "/etc/init.d/network"
#define BACKEND_PATH "knetworkconf/backends/network-conf"

#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <kapp.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <qobject.h>
#include <qdir.h>
#include <qfile.h>
#include <qstring.h>
#include <qprocess.h>
#include <qdom.h>
#include <qlabel.h>
#include <klistbox.h>
#include <kstandarddirs.h>
#include "knetworkinfo.h"
#include "kdetectdistrodlg.h"
#include "kreloadnetworkdlg.h"
#include "kinterfaceupdowndlg.h"
#include "kselectdistrodlg.h"

/**
  *@author Juan Luis Baptiste
  */

class KNetworkConfigParser : public QObject {
  Q_OBJECT
  public: 
	  KNetworkConfigParser();
  	~KNetworkConfigParser();
    /** Returns a list of the configured network interfaces. */
    QPtrList<KNetworkInterface> loadNetworkDevices();
    /** Returns the state of the device, up or down. */
    QString getDeviceState();  /** Reads the info in /etc/sysconfig/network and map it to a KRoutingInfo object. */
    KRoutingInfo *loadNetworkInfo();
  /** Reads /etc/resolv.conf and fills up a KDNSInfo object.
  It also reads /etc/hosts by calling loadKnownHostsInfo() method. */
    KDNSInfo *loadDNSInfo();
  /** return tyhe number of configured devices. */
    unsigned numDevices();
    /** Saves the info of the file /etc/sysconfig/network and backups the acutal file.
    Returns true on success, false on failure.
    Returns true on success, false on failure.*/
    bool saveNetworkInfo(KRoutingInfo *routingInfo);
    /** Creates new ifcfg-[INTERFACE] files in /etc/sysconfig/network-scrpts with the new info,
    and saves backup (.bak) files of the actual configuration files. */
    bool saveDevicesInfo(QPtrList<KNetworkInterface> deviceList);
    /** Saves the DNS info in /etc/resolv.conf. For now it only has support for
    'nameserver' and 'domain' keywords. It saves a backup of the actual file first.
    Returns true on success, false on failure. */
    bool saveDNSInfo(KDNSInfo *dnsInfo);
    /*Saves the /etc/hosts file.*/
    bool saveKnownHostsInfo(QPtrList<KKnownHostInfo> hostList);
    void setProgramVersion(QString ver);
    void loadDhcpDevices();
    void loadDhcpDevice(KNetworkInterface *device);
  /** Reads /proc/net/route looking for the default gateway. */
    void loadRoutingInfo();
  /** Loads the network info contained in netinfo.conf and fills a 
 QPrtList<KNetworkInterface> with the network devices found,
a KDNSInfo with the DNS servers, and a KRoutingInfo with info of
the default gw and routes info.
 */
  KNetworkInfo * loadNetworkInfo2();
  /** Saves the new network configuration. */
  bool saveNetworkInfo2(KNetworkInfo *networkInfo);
  bool saveNetworkInfo3(KNetworkInfo *networkInfo);
  /** Loads the network info from a xml file generated by the gnome system tools
network backends that are included with this app. */
  KNetworkInfo * loadNetworkInfo3();
    QProcess * proc;
	QProcess * procDetect;
  QProcess * procSaveNetworkInfo;
  /** Variable to know if there is an dhcp device configured, so we can know if we need to save the defauilt gw info or not. */
  bool dhcpDevice;
      /** Loads the info of the device in the case that the boot protocol is dhcp or bootp,
      because in this case the information isn't contained in config files. */
    bool loadDhcpInfo(KNetworkInterface *dev);
    /** Calls the network scripts to reload the network depending on the system
    it's running on, so the configuration changes takes effect. */
    bool reloadNetwork();
   /** No descriptions */
   KNetworkInfo * getNewNetworkInfo();

  public slots:
    /** Runs the gst backend to get network values. You can pass to the script a specific platform to load using the platform parameter.  */
    void runDetectionScript(QString platform);
    /** runs gst to find out the state of network devices.It runs the command:
    $knetworkconf_home/backends/networkconf [--platform platform] -d list_ifaces */
    void listIfaces(QString platform);

  private slots:
    void readIfconfigOutput();
    void fillDhcpDevices();
    void fillDhcpDevice(KNetworkInterface *device);
/** Verifies the exit code of the detection shell script to know if it had success or not
    detecting the Linux distro is running on.*/
    void verifyDetectStatusSlot();
    void readNetworkInfo();
    void readFromStdoutReloadScript();
    void closeReloadStatusWindowSlot();
    /** emits a signal when the network changes have benn saved. */
    void sendNetworkInfoSavedSignalSlot();
    /** Concatenates into a QString the xml output of the network backend. */
    void concatXMLOutputSlot();
    void readXMLErrSlot();
    /** Shows the dialog with all the supported platforms by GST. */
    void showSupportedPlatformsDialogSlot();
    /** Parses the xml ouput generated by GST that has all the supported platforms. */
    void readSupportedPlatformsSlot();
    /** No descriptions */
    void assignNewDeviceList();
    void readListIfacesSlot();
    void readFromStdoutSaveNetworkInfo();
  /** Calls runDetectionScript(). */
  void runDetectionScriptSlot();
                
  private:
      QPtrList<KNetworkInterface> deviceList;
  QPtrList<KNetworkInterface>  tempList;
    KNetworkInterface *newDevice;
    bool isUp;
      
  /** Counts the number of configured devices that had been found. */
    unsigned _numDevices;
    KRoutingInfo *routingInfo;
    KDNSInfo *dnsInfo;
    QString programVersion;
    /**  The output of ifconfig command that needs to be parsed in readIfconfigOutput()*/
    QString ifconfigOutput;
  KNetworkInfo * networkInfo;
  QPtrList<KKnownHostInfo> knownHostsList;
    /** Saves a backup of the specified file 'file' with the '.bak' extension. */
    bool backupFile(QString fileName);
    QString hexIPv4ToDecIPv4(QString hex);
  /** Reads /etc/hosts file and returns a QPtrList<KKnownHostInfo>. */
  QPtrList<KKnownHostInfo> loadKnownHostsInfo();
  /** lists all platforms supported by GST. */
  void loadSupportedPlatforms();
  /** Returns the info of a platform in the form of 'key:value' . */
  QString getPlatformInfo(QDomElement platform);

    /** Shows a progress dialog while the networl is being reloaded. */
//    KReloadNetworkDlg * kpdReloadStatusDlg;
    KInterfaceUpDownDlg * kpdReloadStatusDlg;
    QProcess *procReloadNetwork;
    QString reloadScriptOutput;
  /** Contains the network configuration en xml format. */
  /** List of supported platforms. */
  QStringList supportedPlatformsList;
  QString xmlOuput;
  QString xmlErr;
  /** Temporal device list to be used when loading the dhcp devices info */
  QPtrList<KNetworkInterface> tempDevList;
    
  
protected: // Protected methods
  /** Parses all of the <interface>...</interface> entries in the xml configuration file. */
  KNetworkInterface *getInterfaceInfo(QDomElement interface);
  /** Reads the xml with the network info. */
signals: // Signals
  /** emmited when the xml file is finished reading. */
  void readyLoadingNetworkInfo();
  /** emmited when the xml file saved and the backend has finished executing. */
  void readySavingNetworkInfo();  
  /** Signal emmited when the network has finished reloading. */
  void network_reloaded();
  /** Signal that is sent when fillDevices() has finished fllind the dhcp devices. */
  void readyIfconfigOutputSignal();
  /** Signal emmited when the platform can't be auto-detected. */
  void errorDetectingPlatform();  
  /** signal emmited when the parsing of the xml of the supported platform list. */
  void readyLoadingSupportedPlatforms();
};

#endif
